@@ -1,8 +1,7 @@
+use std::future::Future;
 use std::os::raw::{c_char, c_void};
 use std::ptr;
 
-use futures::prelude::*;
-
 use crate::{check_status, sys, Env, JsError, NapiRaw, Result};
 
 pub struct FuturePromise<T, V: NapiRaw> {
